#! /bin/sh

# Get source path.
if [ -z "$SRCPATH" ]; then
	SRCPATH="/usr/src/linux"
	echo "Assuming kernel sources are located at $SRCPATH..."
fi
if [ ! -d "$SRCPATH/include" ]; then
	echo "Source path \"$SRCPATH\" doesn't exist (or it doesn't contain"
	echo "subdirectory \"include\"). This path should point to your"
	echo "kernel sources."
	echo ""
	echo "You can provide path in SRCPATH."
	exit 1
fi

# Get compiler.
if [ -z "$CC" ]; then
	CC=gcc
	echo "Using $CC as compiler."
fi
CC=$(which $CC)

if [ ! -x $CC ]; then
	echo "Cannot run \"$CC\"."
	echo "You can provide compiler in CC."
	exit 1
fi

# Get Linux (source) version.
if [ ! -f "$SRCPATH/include/linux/version.h" ]; then
	echo "Couldn't read \"$SRCPATCH/include/linux/version.h\". Aborting."
	exit 1
fi
if [ -z "$VERSION" ]; then
	VERSION=$(grep UTS_RELEASE $SRCPATH/include/linux/version.h | sed -e 's/.*"\(.*\)".*/\1/')
fi
if [ -z "$VERSION" ]; then
	echo "Couldn't get kernel version."
	echo " You can override version with VERSION."
	exit 1
fi
MINOR=$(echo $VERSION | cut -d '.' -f 2)
if [ -z "MINOR"  ]; then
	echo "Kernel version \"$VERSION\" is bogus, aborting."
	echo "Override version with VERSION."
	exit 1
fi
if [ "$MINOR" -ne "5" -a "$MINOR" -ne "6" ]; then
	echo "Kernel are for linux $VERSION, need 2.6.x or 2.5.x."
	exit 1
fi

# Get gamecon.c
if [ ! -f "$SRCPATH/drivers/input/joystick/gamecon.c" ]; then
	echo "Couldn't find \"$SRCPATH/drivers/input/joystick/gamecon.c\","
	echo "using pre-patched v1.22 instead."
	cp gamecon.c_patched gamecon.c
else
	cp "$SRCPATH/drivers/input/joystick/gamecon.c" .
	patch -s -f <patch
	if [ $? -eq 1 ]; then
		echo "Patching failed, using pre-patched v1.22 instead."
		rm -f gamecon.c
		cp gamecon.c_patched gamecon.c
	fi
fi

	
echo ""
echo "Compiling..."
echo "Don't worry about possible warnings, usually they are harmless."
echo ""

# Get kernel version.
KERNEL_VERSION_FULL=$(uname -sr)
KERNEL_VERSION_NUMBER=$(uname -r)
# ...and module path.
MODULE_PATH="/lib/modules/$KERNEL_VERSION_NUMBER/kernel/drivers/input/joystick"

make -C $SRCPATH SUBDIRS=$(pwd) modules

if [ $? -eq 1 ]; then
	echo ""
	echo "Compiling failed."
	echo ""
	exit 1
fi

cat <<EOF

All done, you should have gamecon.ko in your working directory. This
module *should* be able to replace your stock-gamecon, but if I was you,
I would try loading it with insmod before replacing the original one
(even then I would make a backup of the old one).

If you think everything went fine, go ahead and type
cp gamecon.ko $MODULE_PATH
and answer "yes" if requested to confirm file overwrite.

Whatever you do, remember it's (only) the kernel you're playing with. :-)

EOF
